package dev.shadowsoffire.apotheosis.adventure;

import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.crafting.Ingredient;
import net.minecraft.world.item.crafting.SmithingTransformRecipe;
import java.util.HashMap;
import org.apache.logging.log4j.LogManager;
import dev.shadowsoffire.placebo.recipe.RecipeHelper;
import net.minecraft.world.item.Item;
import java.util.Locale;
import net.minecraftforge.fml.InterModComms;
import net.minecraft.core.BlockPos;
import dev.shadowsoffire.apotheosis.util.NameHelper;
import dev.shadowsoffire.placebo.config.Configuration;
import java.io.File;
import net.minecraftforge.fml.event.lifecycle.InterModProcessEvent;
import net.minecraftforge.fml.javafmlmod.FMLJavaModLoadingContext;
import dev.shadowsoffire.apotheosis.adventure.client.AdventureModuleClient;
import net.minecraftforge.fml.event.lifecycle.FMLClientSetupEvent;
import dev.shadowsoffire.apotheosis.adventure.gen.BlacklistModifier;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixHookLootModifier;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixConvertLootModifier;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixLootModifier;
import dev.shadowsoffire.apotheosis.adventure.loot.GemLootModifier;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraftforge.registries.RegisterEvent;
import dev.shadowsoffire.apotheosis.adventure.affix.reforging.ReforgingRecipe;
import dev.shadowsoffire.apotheosis.adventure.affix.salvaging.SalvagingRecipe;
import dev.shadowsoffire.apotheosis.adventure.socket.AddSocketsRecipe;
import net.minecraft.world.item.crafting.RecipeSerializer;
import dev.shadowsoffire.apotheosis.adventure.affix.augmenting.AugmentingTableTile;
import com.mojang.datafixers.types.Type;
import dev.shadowsoffire.apotheosis.adventure.affix.salvaging.SalvagingTableTile;
import dev.shadowsoffire.apotheosis.adventure.affix.reforging.ReforgingTableTile;
import java.util.Set;
import dev.shadowsoffire.placebo.block_entity.TickingBlockEntityType;
import com.google.common.collect.ImmutableSet;
import dev.shadowsoffire.apotheosis.adventure.boss.BossSpawnerBlock;
import net.minecraft.world.level.block.entity.BlockEntityType;
import dev.shadowsoffire.placebo.registry.RegistryEvent;
import dev.shadowsoffire.placebo.tabs.TabFillingRegistry;
import java.util.function.Supplier;
import dev.shadowsoffire.apotheosis.util.GemIngredient;
import net.minecraftforge.common.crafting.IIngredientSerializer;
import net.minecraftforge.common.crafting.CraftingHelper;
import dev.shadowsoffire.apotheosis.util.AffixItemIngredient;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.apotheosis.adventure.boss.Exclusion;
import dev.shadowsoffire.apotheosis.adventure.loot.GemLootPoolEntry;
import net.minecraft.core.Registry;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixLootPoolEntry;
import net.minecraft.core.registries.BuiltInRegistries;
import dev.shadowsoffire.placebo.loot.LootSystem;
import net.minecraft.world.level.block.Block;
import dev.shadowsoffire.apotheosis.adventure.compat.AdventureTwilightCompat;
import dev.shadowsoffire.apotheosis.adventure.compat.AdventureTOPPlugin;
import dev.shadowsoffire.apotheosis.adventure.compat.GatewaysCompat;
import net.minecraftforge.fml.ModList;
import dev.shadowsoffire.apotheosis.adventure.socket.UnnamingRecipe;
import dev.shadowsoffire.apotheosis.adventure.socket.WithdrawalRecipe;
import net.minecraft.world.item.crafting.Recipe;
import dev.shadowsoffire.apotheosis.adventure.socket.SocketingRecipe;
import dev.shadowsoffire.apotheosis.adventure.boss.MinibossRegistry;
import dev.shadowsoffire.apotheosis.adventure.spawner.RogueSpawnerRegistry;
import dev.shadowsoffire.apotheosis.adventure.boss.BossRegistry;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixLootRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixRegistry;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import java.util.function.Consumer;
import dev.shadowsoffire.apotheosis.adventure.boss.BossEvents;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fml.event.lifecycle.FMLCommonSetupEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.util.ObfuscationReflectionHelper;
import net.minecraft.world.entity.ai.attributes.Attributes;
import net.minecraft.world.entity.ai.attributes.RangedAttribute;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import net.minecraft.resources.ResourceLocation;
import java.util.Map;
import org.apache.logging.log4j.Logger;

public class AdventureModule
{
    public static final Logger LOGGER;
    public static final boolean STAGES_LOADED;
    static final Map<ResourceLocation, LootCategory> IMC_TYPE_OVERRIDES;
    public static final boolean DEBUG = false;
    
    public AdventureModule() {
        Adventure.bootstrap();
    }
    
    @SubscribeEvent
    public void preInit(final Apotheosis.ApotheosisConstruction e) {
        ObfuscationReflectionHelper.setPrivateValue((Class)RangedAttribute.class, (Object)Attributes.f_22284_, (Object)200.0, "f_22308_");
        ObfuscationReflectionHelper.setPrivateValue((Class)RangedAttribute.class, (Object)Attributes.f_22285_, (Object)100.0, "f_22308_");
    }
    
    @SubscribeEvent
    public void init(final FMLCommonSetupEvent e) {
        this.reload(null);
        MinecraftForge.EVENT_BUS.register((Object)new AdventureEvents());
        MinecraftForge.EVENT_BUS.register((Object)new BossEvents());
        MinecraftForge.EVENT_BUS.addListener((Consumer)this::reload);
        RarityRegistry.INSTANCE.registerToBus();
        AffixRegistry.INSTANCE.registerToBus();
        GemRegistry.INSTANCE.registerToBus();
        AffixLootRegistry.INSTANCE.registerToBus();
        BossRegistry.INSTANCE.registerToBus();
        RogueSpawnerRegistry.INSTANCE.registerToBus();
        MinibossRegistry.INSTANCE.registerToBus();
        Apotheosis.HELPER.registerProvider(f -> {
            f.addRecipe((Recipe)new SocketingRecipe());
            f.addRecipe((Recipe)new WithdrawalRecipe());
            f.addRecipe((Recipe)new UnnamingRecipe());
            return;
        });
        e.enqueueWork(() -> {
            if (ModList.get().isLoaded("gateways")) {
                GatewaysCompat.register();
            }
            if (ModList.get().isLoaded("theoneprobe")) {
                AdventureTOPPlugin.register();
            }
            if (ModList.get().isLoaded("twilightforest")) {
                AdventureTwilightCompat.register();
            }
            LootSystem.defaultBlockTable((Block)Adventure.Blocks.SIMPLE_REFORGING_TABLE.get());
            LootSystem.defaultBlockTable((Block)Adventure.Blocks.REFORGING_TABLE.get());
            LootSystem.defaultBlockTable((Block)Adventure.Blocks.SALVAGING_TABLE.get());
            LootSystem.defaultBlockTable((Block)Adventure.Blocks.GEM_CUTTING_TABLE.get());
            LootSystem.defaultBlockTable((Block)Adventure.Blocks.AUGMENTING_TABLE.get());
            Registry.m_122965_(BuiltInRegistries.f_257035_, new ResourceLocation("apotheosis", "random_affix_item"), (Object)AffixLootPoolEntry.TYPE);
            Registry.m_122965_(BuiltInRegistries.f_257035_, new ResourceLocation("apotheosis", "random_gem"), (Object)GemLootPoolEntry.TYPE);
            Exclusion.initSerializers();
            GemBonus.initCodecs();
            CraftingHelper.register(Apotheosis.loc("affix_item"), (IIngredientSerializer)AffixItemIngredient.Serializer.INSTANCE);
            CraftingHelper.register(Apotheosis.loc("gem"), (IIngredientSerializer)GemIngredient.Serializer.INSTANCE);
            TabFillingRegistry.register(Adventure.Tabs.ADVENTURE.getKey(), new Supplier[] { (Supplier)Adventure.Items.COMMON_MATERIAL, (Supplier)Adventure.Items.UNCOMMON_MATERIAL, (Supplier)Adventure.Items.RARE_MATERIAL, (Supplier)Adventure.Items.EPIC_MATERIAL, (Supplier)Adventure.Items.MYTHIC_MATERIAL, (Supplier)Adventure.Items.GEM_DUST, (Supplier)Adventure.Items.GEM_FUSED_SLATE, (Supplier)Adventure.Items.SIGIL_OF_SOCKETING, (Supplier)Adventure.Items.SIGIL_OF_WITHDRAWAL, (Supplier)Adventure.Items.SIGIL_OF_REBIRTH, (Supplier)Adventure.Items.SIGIL_OF_ENHANCEMENT, (Supplier)Adventure.Items.SIGIL_OF_UNNAMING, (Supplier)Adventure.Items.BOSS_SUMMONER, (Supplier)Adventure.Items.SALVAGING_TABLE, (Supplier)Adventure.Items.GEM_CUTTING_TABLE, (Supplier)Adventure.Items.SIMPLE_REFORGING_TABLE, (Supplier)Adventure.Items.REFORGING_TABLE, (Supplier)Adventure.Items.AUGMENTING_TABLE });
            TabFillingRegistry.register(Adventure.Tabs.ADVENTURE.getKey(), new Supplier[] { (Supplier)Adventure.Items.GEM });
        });
    }
    
    @SubscribeEvent
    public void tiles(final RegistryEvent.Register<BlockEntityType<?>> e) {
        e.getRegistry().register((Object)new TickingBlockEntityType(BossSpawnerBlock.BossSpawnerTile::new, (Set)ImmutableSet.of(Adventure.Blocks.BOSS_SPAWNER.get()), false, true), "boss_spawner");
        e.getRegistry().register((Object)new TickingBlockEntityType(ReforgingTableTile::new, (Set)ImmutableSet.of(Adventure.Blocks.SIMPLE_REFORGING_TABLE.get(), Adventure.Blocks.REFORGING_TABLE.get()), true, false), "reforging_table");
        e.getRegistry().register((Object)new BlockEntityType(SalvagingTableTile::new, (Set)ImmutableSet.of(Adventure.Blocks.SALVAGING_TABLE.get()), (Type)null), "salvaging_table");
        e.getRegistry().register((Object)new TickingBlockEntityType(AugmentingTableTile::new, (Set)ImmutableSet.of(Adventure.Blocks.AUGMENTING_TABLE.get()), true, false), "augmenting_table");
    }
    
    @SubscribeEvent
    public void serializers(final RegistryEvent.Register<RecipeSerializer<?>> e) {
        e.getRegistry().register((Object)SocketingRecipe.Serializer.INSTANCE, "socketing");
        e.getRegistry().register((Object)WithdrawalRecipe.Serializer.INSTANCE, "widthdrawal");
        e.getRegistry().register((Object)UnnamingRecipe.Serializer.INSTANCE, "unnaming");
        e.getRegistry().register((Object)AddSocketsRecipe.Serializer.INSTANCE, "add_sockets");
        e.getRegistry().register((Object)SalvagingRecipe.Serializer.INSTANCE, "salvaging");
        e.getRegistry().register((Object)ReforgingRecipe.Serializer.INSTANCE, "reforging");
    }
    
    @SubscribeEvent
    public void miscRegistration(final RegisterEvent e) {
        if (e.getForgeRegistry() == ForgeRegistries.GLOBAL_LOOT_MODIFIER_SERIALIZERS.get()) {
            e.getForgeRegistry().register("gems", (Object)GemLootModifier.CODEC);
            e.getForgeRegistry().register("affix_loot", (Object)AffixLootModifier.CODEC);
            e.getForgeRegistry().register("affix_conversion", (Object)AffixConvertLootModifier.CODEC);
            e.getForgeRegistry().register("affix_hook", (Object)AffixHookLootModifier.CODEC);
        }
        if (e.getForgeRegistry() == ForgeRegistries.BIOME_MODIFIER_SERIALIZERS.get()) {
            e.getForgeRegistry().register("blacklist", (Object)BlacklistModifier.CODEC);
        }
    }
    
    @SubscribeEvent
    public void client(final FMLClientSetupEvent e) {
        e.enqueueWork(AdventureModuleClient::init);
        FMLJavaModLoadingContext.get().getModEventBus().register((Object)new AdventureModuleClient());
    }
    
    @SubscribeEvent
    public void imc(final InterModProcessEvent e) {
        // 
        // This method could not be decompiled.
        // 
        // Original Bytecode:
        // 
        //     1: invokevirtual   net/minecraftforge/fml/event/lifecycle/InterModProcessEvent.getIMCStream:()Ljava/util/stream/Stream;
        //     4: invokedynamic   BootstrapMethod #8, accept:()Ljava/util/function/Consumer;
        //     9: invokeinterface java/util/stream/Stream.forEach:(Ljava/util/function/Consumer;)V
        //    14: return         
        // 
        // The error that occurred was:
        // 
        // java.lang.NullPointerException: Cannot invoke "com.strobel1.assembler.metadata.TypeReference.getSimpleType()" because the return value of "com.strobel1.decompiler.ast.Variable.getType()" is null
        //     at com.strobel1.decompiler.languages.java.ast.NameVariables.generateNameForVariable(NameVariables.java:252)
        //     at com.strobel1.decompiler.languages.java.ast.NameVariables.assignNamesToVariables(NameVariables.java:185)
        //     at com.strobel1.decompiler.languages.java.ast.AstMethodBodyBuilder.nameVariables(AstMethodBodyBuilder.java:1482)
        //     at com.strobel1.decompiler.languages.java.ast.AstMethodBodyBuilder.populateVariables(AstMethodBodyBuilder.java:1411)
        //     at com.strobel1.decompiler.languages.java.ast.AstMethodBodyBuilder.createMethodBody(AstMethodBodyBuilder.java:210)
        //     at com.strobel1.decompiler.languages.java.ast.AstMethodBodyBuilder.createMethodBody(AstMethodBodyBuilder.java:93)
        //     at com.strobel1.decompiler.languages.java.ast.AstBuilder.createMethodBody(AstBuilder.java:868)
        //     at com.strobel1.decompiler.languages.java.ast.AstBuilder.createMethod(AstBuilder.java:761)
        //     at com.strobel1.decompiler.languages.java.ast.AstBuilder.addTypeMembers(AstBuilder.java:638)
        //     at com.strobel1.decompiler.languages.java.ast.AstBuilder.createTypeCore(AstBuilder.java:605)
        //     at com.strobel1.decompiler.languages.java.ast.AstBuilder.createTypeNoCache(AstBuilder.java:195)
        //     at com.strobel1.decompiler.languages.java.ast.AstBuilder.createType(AstBuilder.java:162)
        //     at com.strobel1.decompiler.languages.java.ast.AstBuilder.addType(AstBuilder.java:137)
        //     at com.strobel1.decompiler.languages.java.JavaLanguage.buildAst(JavaLanguage.java:71)
        //     at com.strobel1.decompiler.languages.java.JavaLanguage.decompileType(JavaLanguage.java:59)
        //     at com.strobel1.decompiler.DecompilerDriver.decompileType(DecompilerDriver.java:334)
        //     at com.strobel1.decompiler.DecompilerDriver.decompileJar(DecompilerDriver.java:255)
        //     at com.strobel1.decompiler.DecompilerDriver.main(DecompilerDriver.java:144)
        //     at javare.DeMain.runProcyon(DeMain.java:110)
        //     at javare.DeMain.run(DeMain.java:60)
        //     at javare.DeMain.main(DeMain.java:32)
        //     at client.StatusTrackingProcessor.processMessage(StatusTrackingProcessor.java:87)
        //     at client.JarSplitterStreamsConsumer.lambda$0(JarSplitterStreamsConsumer.java:60)
        //     at org.apache.kafka.streams.kstream.ForeachProcessor.process(ForeachProcessor.java:32)
        //     at org.apache.kafka.streams.processor.internals.ProcessorNode.process(ProcessorNode.java:152)
        //     at org.apache.kafka.streams.processor.internals.ProcessorContextImpl.forwardInternal(ProcessorContextImpl.java:291)
        //     at org.apache.kafka.streams.processor.internals.ProcessorContextImpl.forward(ProcessorContextImpl.java:270)
        //     at org.apache.kafka.streams.processor.internals.ProcessorContextImpl.forward(ProcessorContextImpl.java:229)
        //     at org.apache.kafka.streams.processor.internals.SourceNode.process(SourceNode.java:84)
        //     at org.apache.kafka.streams.processor.internals.StreamTask.lambda$doProcess$1(StreamTask.java:847)
        //     at org.apache.kafka.streams.processor.internals.metrics.StreamsMetricsImpl.maybeMeasureLatency(StreamsMetricsImpl.java:872)
        //     at org.apache.kafka.streams.processor.internals.StreamTask.doProcess(StreamTask.java:847)
        //     at org.apache.kafka.streams.processor.internals.StreamTask.process(StreamTask.java:778)
        //     at org.apache.kafka.streams.processor.internals.TaskExecutor.processTask(TaskExecutor.java:97)
        //     at org.apache.kafka.streams.processor.internals.TaskExecutor.process(TaskExecutor.java:78)
        //     at org.apache.kafka.streams.processor.internals.TaskManager.process(TaskManager.java:1944)
        //     at org.apache.kafka.streams.processor.internals.StreamThread.runOnceWithoutProcessingThreads(StreamThread.java:953)
        //     at org.apache.kafka.streams.processor.internals.StreamThread.runLoop(StreamThread.java:686)
        //     at org.apache.kafka.streams.processor.internals.StreamThread.run(StreamThread.java:645)
        // 
        throw new IllegalStateException("An error occurred while decompiling this method.");
    }
    
    public void reload(final Apotheosis.ApotheosisReloadEvent e) {
        final Configuration mainConfig = new Configuration(new File(Apotheosis.configDir, "adventure.cfg"));
        final Configuration nameConfig = new Configuration(new File(Apotheosis.configDir, "names.cfg"));
        AdventureConfig.load(mainConfig);
        NameHelper.load(nameConfig);
        if (e == null && mainConfig.hasChanged()) {
            mainConfig.save();
        }
        if (e == null && nameConfig.hasChanged()) {
            nameConfig.save();
        }
    }
    
    public static void debugLog(final BlockPos pos, final String name) {
    }
    
    static {
        LOGGER = LogManager.getLogger("Apotheosis : Adventure");
        STAGES_LOADED = ModList.get().isLoaded("gamestages");
        IMC_TYPE_OVERRIDES = new HashMap<ResourceLocation, LootCategory>();
    }
    
    public static class ApothSmithingRecipe extends SmithingTransformRecipe
    {
        public static final int TEMPLATE = 0;
        public static final int BASE = 1;
        public static final int ADDITION = 2;
        
        public ApothSmithingRecipe(final ResourceLocation pId, final Ingredient pBase, final Ingredient pAddition, final ItemStack pResult) {
            super(pId, Ingredient.f_43901_, pBase, pAddition, pResult);
        }
        
        public boolean m_266343_(final ItemStack pStack) {
            return !LootCategory.forItem(pStack).isNone();
        }
    }
}
